#version 150

uniform sampler2D tex0;
uniform sampler2D tex1;
uniform sampler2D tex2;

in vec2 tex;

out vec4 color;

// From http://rastergrid.com/blog/2010/09/efficient-gaussian-blur-with-linear-sampling/
uniform float offset[3] = float[]( 0.0, 1.3846153846, 3.2307692308 );
uniform float weight[3] = float[]( 0.2270270270, 0.3162162162, 0.0702702703 );

uniform int screen_height;
uniform int screen_width;
uniform float black_point;
uniform float white_point;

const float near = 0.1;
const float far = 1000.0;
const float _rad = 15.0;
const float _threshold = 0.045;
uniform vec3 sphere_points[16] = vec3[](vec3(0, -1, 0), vec3(1, 0, 0), vec3(0, 0, 1), vec3(0.111, -0.111, 0.111), 
										vec3(0.707, -0.707, 0), vec3(0.707, 0, 0.707), vec3(0, -0.707, 0.707),  vec3(0, -0.707, -0.707), 
										vec3(-1, 0, 0), vec3(0, 0, -1), vec3(-0.707, -0.707, 0), vec3(-0.707, 0, -0.707),
										vec3(-0.707, -0.707, 0), vec3(0.707, 0, -0.707), vec3(0.111, -0.111, -0.111),  vec3(-0.111, 0.111, 0.111) );
/// you can edit these ///
const bool SSAO = true; //change this to false if you don't want SSAO
const bool DOF = true; //likewise if you don't want DOF
const bool BOKEH = false; //I've left this off in alpha 211 because the bloom makes it a bit redundant
const bool DOFDT = true; //The blur by default tests the depth to make sure closer objects aren't blurred into futher ones. This turns it off if it uses too much power.
const float DOF_DEPTH = 0.7; //I reccomend from 0.2 to 1.9, higher is blurrier, doesn't change GPU usage
const float SSAO_FAR = 0.6; //controls how near something has to be before SSAO is calculated. from 0 to 1. Higher requires more GPU power, but it's not that taxing really
const int DOF_QUALITY = 2; //1 = low, 2 = medium, 3 = high; Lower number requires lower GPU power. If it's getting laggy, put this down or turn DOF to false
const int SSAO_QUALITY = 3; //same deal as DOF but for SSAO
/// you can edit these ///


float DistFromDepth(float depth) {   
    return (near) / (far - depth * (far - near)) * far;
}

void TestOcclusion(in float distance, in float thickness, in vec2 test_point, inout float occlusion, inout float divisor, in vec2 pixel_size){
    float depth = texture2D( tex1, test_point ).r;
    float temp_distance = DistFromDepth(depth);

    if(temp_distance < distance) {
		float occlusion_delta = temp_distance/distance;

		if (distance-temp_distance < thickness/distance) {
			occlusion += occlusion_delta;
		}
		divisor += occlusion_delta;
    } else {
        divisor += 1.0;
    }
}

void TestOcclusionCircle(in float thickness, in float radius, in float distance, in vec2 tex_point, inout float occlusion, inout float divisor, in vec3 noise, in vec2 pixel_size){
    for(int i=0; i<16; ++i){
        vec3 sp = reflect(sphere_points[i], noise);
        TestOcclusion(distance + sp.z * 0.002 * radius, thickness, tex_point + sp.xy * radius * pixel_size, occlusion, divisor, pixel_size);
    }
}

void main(void)
{
#if defined(BLUR_VERT)
    vec4 FragmentColor;
    float pixel_height = 1.0 / float(screen_height);
    FragmentColor = texture( tex0, tex ) * weight[0];
    for (int i=1; i<3; i++) {
        FragmentColor +=
            texture( tex0, tex+vec2(0.0, offset[i]*pixel_height) )
                * weight[i];
        FragmentColor +=
            texture( tex0, tex-vec2(0.0, offset[i]*pixel_height) )
                * weight[i];
    }
    color = FragmentColor;
#elif defined(BLUR_HORZ)
    vec4 FragmentColor;
    float pixel_width = 1.0 / float(screen_width);
    FragmentColor = texture( tex0, tex ) * weight[0];
    for (int i=1; i<3; i++) {
        FragmentColor +=
            texture( tex0, tex+vec2(offset[i]*pixel_width, 0.0) )
                * weight[i];
        FragmentColor +=
            texture( tex0, tex-vec2(offset[i]*pixel_width, 0.0) )
                * weight[i];
    }
    color = FragmentColor;
#elif defined(OVERBRIGHT)
    color = max(vec4(0.0), texture( tex0, tex ) - vec4(1.0));
#elif defined(DOWNSAMPLE)
    float pixel_width = 1.0 / float(screen_width);
    float pixel_height = 1.0 / float(screen_height);
    color = texture( tex0, tex + vec2( pixel_width,  pixel_height)) +
                   texture( tex0, tex + vec2( pixel_width, -pixel_height)) +
                   texture( tex0, tex + vec2(-pixel_width,  pixel_height)) +
                   texture( tex0, tex + vec2(-pixel_width, -pixel_height));
   color *= 0.25;
#elif defined(TONE_MAP)
    float temp_wp = 0.7;
    float temp_bp = 0.005;
    float contrast = 1.0 / (temp_wp - temp_bp);
    color = max(vec4(0.0), (texture(tex0, tex) - vec4(temp_bp)) * contrast);    
#elif defined(ADD)
    color = texture(tex1, tex) + mix(texture(tex0, tex) , texture(tex2, tex), 0.75);
    vec3 overbright = max(vec3(0.0), color.xyz - vec3(1.0));
    float avg = (overbright[0] + overbright[1] + overbright[2]) / 3.0;
    color.xyz = mix(color.xyz, vec3(1.0), min(1.0,avg*0.3));
#else
    vec3 color_map = texture2D(tex0, tex).rgb;
    float depth = texture2D(tex1, tex).r;
    vec3 noise = normalize((texture2D( tex2,tex).rgb )- vec3(0.5));
    noise *= 0.4;
    float distance = DistFromDepth(depth);
    
    #ifdef pixel_width
    #else
    float pixel_width = 1.0 / float(screen_width);
    #endif 
    #ifdef pixel_height
    #else
    float pixel_height = 1.0 / float(screen_height);
    #endif 
    
	vec2 pixel_size = vec2(pixel_width, pixel_height);
	//133
	float focus = min(0.99, texture2D( tex1, vec2(0.5, 0.5)).r);
    
    float r = pow(abs(depth - focus), 1.65)*140.0*DOF_DEPTH;
    float divisor = 0.0;
    
	vec3 blur_colour = vec3(0.0);
	vec3 bokeh = vec3(0.0);
	
	float ssaofar = 0.98+SSAO_FAR*0.03;
	
    if (DOF) {
		vec2 xy;
		
		if (BOKEH) {
			vec3 test;
			float testdep;
			bokeh = color_map;
			
			int x,y;
			x = 0;
			y = 0;
			
			
			for (x=-3; x<3; x+=1){
				for ( y=-3; y<3; y+=1){
					if(length(vec2(float(x),float(y)))<2.8){
						xy = vec2(float(x),float(y))*pixel_size;
						test = min(vec3(0.99), texture2D( tex0, tex + xy).rgb);
						testdep = min(0.99, texture2D( tex1, tex + xy).r);
						if ((test.r+test.g+test.b)*pow(abs(testdep - focus), 2.2)*140.0 > bokeh.r+bokeh.g+bokeh.b) {
							bokeh = test;
						}
					}
				}
			}
			
			bokeh -= vec3(0.3);
			
			bokeh = max(vec3(0.0), bokeh);
			blur_colour += bokeh;
		}
		
		
		for (int x = -2*DOF_QUALITY; x < 2*DOF_QUALITY; x++) {
			for (int y = -2*DOF_QUALITY; y < 2*DOF_QUALITY; y++) {
				xy = vec2(float(x), float(y))*pixel_size*r;
				if (DOFDT && length(xy) < r) {
					if (texture2D(tex1, tex+xy).r >= depth) {
						blur_colour += texture2D(tex0, tex+xy).rgb;
						divisor++;
					}
				}
			}	
		}
		
		divisor = max(0.0, divisor);
		
		blur_colour *= vec3(1.0/divisor);
		
	} else {
		blur_colour = color_map;
	}
	
	color_map = blur_colour+bokeh;
	float occlusion = 0.0;
    if (SSAO && depth < ssaofar) {
		
		divisor = 0.0;
		if (SSAO_QUALITY > 2.0) {
			TestOcclusionCircle(1.2, min(64.0,max(16.0,512.0/distance)), distance, tex, occlusion, divisor, noise, pixel_size);
		} 
		if (SSAO_QUALITY > 1.0) {
			TestOcclusionCircle(0.9, min(24.0,max(16.0,440.0/distance)), distance, tex, occlusion, divisor, -noise, pixel_size);
		}
		if (SSAO_QUALITY > 0.0) {
			TestOcclusionCircle(0.3, min(16.0,max(2.0,240.0/distance)), distance, tex, occlusion, divisor, noise, pixel_size); 
		}
		occlusion /= divisor;
    
		occlusion = 1.0-occlusion;
		occlusion = min(occlusion, 0.5)*2.0;
		color_map *= mix(occlusion, 1.0, max(0.0, 1.0-(ssaofar-depth)*50.0)); 	}

    color = vec4(color_map, texture2D(tex0, tex).a);
#endif
}
